package cd.casic.ci.common.pipeline.pojo.time;

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Getter;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.pojo.time
 * @Project：ops-pro
 * @name：BuildTimestampType
 * @Date：2025/03/26 11:12
 * @Filename：BuildTimestampType
 * @description：Todo
 */
@Getter
@AllArgsConstructor
@Schema(title = "构建详情记录-时间戳类型（勿随意删除）")
public enum BuildTimestampType {

    BUILD_REVIEW_WAITING("buildReviewWaiting"), // 流水线触发审核等待
    BUILD_CONCURRENCY_QUEUE("buildConcurrencyQueue"), // 流水线并发排队
    STAGE_CHECK_IN_WAITING("stageCheckInWaiting"), // stage准入等待
    STAGE_CHECK_OUT_WAITING("stageCheckOutWaiting"), // stage准出等待
    JOB_MUTEX_QUEUE("jobMutexQueue"), // job互斥并发排队
    JOB_AGENT_REUSE_MUTEX_QUEUE("jobAgentReuseMutexQueue"), // jobAgent复用互斥排队
    JOB_THIRD_PARTY_QUEUE("jobThirdPartyQueue"), // job第三方构建机资源排队
    JOB_CONTAINER_STARTUP("jobContainerStartup"), // job构建机启动（包含了第三方构建机资源等待）
    JOB_CONTAINER_SHUTDOWN("jobContainerShutdown"), // job构建机关闭
    TASK_REVIEW_PAUSE_WAITING("taskReviewPauseWaiting"); // task等待（包括插件暂停、人工审核、质量红线审核）

    private final String action;

    // 使插件处于等待的类型
    public boolean taskCheckWait() {
        return this == TASK_REVIEW_PAUSE_WAITING;
    }

    // 使container处于排队的类型
    public boolean containerCheckQueue() {
        return this == JOB_MUTEX_QUEUE || this == JOB_THIRD_PARTY_QUEUE || this == JOB_AGENT_REUSE_MUTEX_QUEUE;
    }

    // 使stage处于等待的类型
    public boolean stageCheckWait() {
        return this == STAGE_CHECK_IN_WAITING || this == STAGE_CHECK_OUT_WAITING;
    }
}
