package cd.casic.ci.process.api.process.pojo.pipeline.record;

import cd.casic.ci.common.pipeline.enums.BuildRecordTimeStamp;
import cd.casic.ci.common.pipeline.enums.EnvControlTaskType;
import cd.casic.ci.common.pipeline.pojo.element.Element;
import cd.casic.ci.common.pipeline.pojo.element.ElementPostInfo;
import cd.casic.ci.common.pipeline.pojo.time.BuildTimestampType;
import cd.casic.ci.process.api.engine.pojo.PipelineBuildTask;
import cn.hutool.json.JSONUtil;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author by mianbin
 * @Classname BuildRecordTask
 * @Description TODO
 * @Date 2025/3/31 17:19
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@Schema(title = "构建详情记录-插件任务")
public class BuildRecordTask {
    @Schema(title = "构建ID", required = true)
    private String buildId;
    @Schema(title = "项目ID", required = true)
    private String projectId;
    @Schema(title = "流水线ID", required = true)
    private String pipelineId;
    @Schema(title = "编排版本号", required = true)
    private int resourceVersion;
    @Schema(title = "步骤ID", required = true)
    private String stageId;
    @Schema(title = "作业容器ID", required = true)
    private String containerId;
    @Schema(title = "任务ID", required = true)
    private String taskId;
    @Schema(title = "任务序号", required = true)
    private int taskSeq;
    @Schema(title = "执行次数", required = true)
    private int executeCount;
    @Schema(title = "执行变量", required = true)
    private Map<String, Object> taskVar = new HashMap<>();
    @Schema(title = "插件post信息", required = false)
    private ElementPostInfo elementPostInfo;
    @Schema(title = "插件类型标识", required = true)
    private String classType;
    @Schema(title = "市场插件标识", required = true)
    private String atomCode;
    @Schema(title = "构建状态", required = false)
    private String status;
    @Schema(title = "分裂前原类型标识", required = false)
    private String originClassType;
    @Schema(title = "开始时间", required = true)
    private LocalDateTime startTime;
    @Schema(title = "结束时间", required = true)
    private LocalDateTime endTime;
    @Schema(title = "业务时间戳集合", required = true)
    private Map<BuildTimestampType, BuildRecordTimeStamp> timestamps = new HashMap<>();
    @Schema(title = "异步执行状态", required = true)
    private String asyncStatus;

    public  BuildRecordTask(String projectId, String pipelineId, String buildId, String stageId, String id, String id1, String classType, String atomCode, int executeCount, int resourceVersion, int taskSeq, String s, Map<String, Object> stringObjectMap, Map<BuildTimestampType, BuildRecordTimeStamp> kvHashMap, ElementPostInfo elementPostInfo) {
        this.buildId = buildId;
        this.projectId = projectId;
        this.pipelineId = pipelineId;
        this.resourceVersion = resourceVersion;
        this.stageId = stageId;
        this.containerId = id;
        this.taskId = id1;
        this.taskSeq = taskSeq;
        this.executeCount = executeCount;
        this.taskVar = stringObjectMap;
        this.timestamps = kvHashMap;
        this.classType = classType;
        this.atomCode = atomCode;
        this.status = s;
        this.elementPostInfo = elementPostInfo;
        this.startTime = LocalDateTime.now();
        this.endTime = LocalDateTime.now();
    }

    public static class BuildRecordTaskHelper {
        public static void addRecords(List<BuildRecordTask> buildRecordTasks, List<PipelineBuildTask> buildTaskList, int resourceVersion) {
            for (PipelineBuildTask task : buildTaskList) {
                // 自动填充的构建机控制插件，不需要存入Record
                if (EnvControlTaskType.parse(task.getTaskType()) != null) {
                    continue;
                }
                Element element = JSONUtil.toBean(JSONUtil.toJsonStr(task.getTaskParams()), Element.class);
                BuildRecordTask recordTask = new BuildRecordTask();
                recordTask.setProjectId(task.getProjectId());
                recordTask.setPipelineId(task.getPipelineId());
                recordTask.setBuildId(task.getBuildId());
                recordTask.setStageId(task.getStageId());
                recordTask.setContainerId(task.getContainerId());
                recordTask.setTaskSeq(task.getTaskSeq());
                recordTask.setTaskId(task.getTaskId());
                recordTask.setClassType(task.getTaskType());
                recordTask.setAtomCode(task.getAtomCode() != null ? task.getAtomCode() : task.getTaskAtom());
                recordTask.setExecuteCount(task.getExecuteCount() != null ? task.getExecuteCount() : 1);
                recordTask.setResourceVersion(resourceVersion);
                recordTask.setTaskVar(element.initTaskVar());
                recordTask.setTimestamps(new HashMap<>());
                if (task.getAdditionalOptions() != null && task.getAdditionalOptions().getElementPostInfo() != null
                        && !task.getAdditionalOptions().getElementPostInfo().getParentElementId().equals(task.getTaskId())) {
                    recordTask.setElementPostInfo(task.getAdditionalOptions().getElementPostInfo());
                }
                recordTask.setStartTime(task.getStartTime());
                recordTask.setEndTime(task.getEndTime());
                buildRecordTasks.add(recordTask);
            }
        }
    }
}
