package cd.casic.pipeline.store.api.common;


import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.enums.StoreTypeEnum;
import cd.casic.pipeline.store.pojo.common.env.StorePkgRunEnvInfo;
import cd.casic.pipeline.store.pojo.common.sensitive.SensitiveConfResp;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.*;


@Tag(name = "BUILD_STORE", description = "build-store")
@RestController
@RequestMapping("/build/store/")
public interface BuildStoreResource {

    @Operation(summary = "获取敏感数据")
    @GetMapping("/sensitiveConf/types/{storeType}/codes/{storeCode}")
    CommonResult<List<SensitiveConfResp>> getSensitiveConf(
            @RequestHeader(AUTH_HEADER_DEVOPS_BUILD_ID)
            @Parameter(description = "构建ID", required = true)
            String buildId,
            @RequestHeader(AUTH_HEADER_DEVOPS_VM_SEQ_ID)
            @Parameter(description = "构建环境ID", required = true)
            String vmSeqId,
            @PathVariable("storeType")
            @Parameter(description = "组件类型", required = true)
            StoreTypeEnum storeType,
            @PathVariable("storeCode")
            @Parameter(description = "组件标识", required = true)
            String storeCode
    );

    @Operation(summary = "获取组件安装包运行时环境信息")
    @GetMapping("/pkg/envs/types/{storeType}/languages/{language}/versions/{runtimeVersion}/get")
    CommonResult<StorePkgRunEnvInfo> getStorePkgRunEnvInfo(
            @RequestHeader(AUTH_HEADER_DEVOPS_ENV)
            @Parameter(description = "环境信息", required = false)
            String devopsEnv,
            @PathVariable("storeType")
            @Parameter(description = "组件类型", required = true)
            StoreTypeEnum storeType,
            @PathVariable("language")
            @Parameter(description = "开发语言", required = true)
            String language,
            @PathVariable("runtimeVersion")
            @Parameter(description = "运行时版本", required = true)
            String runtimeVersion,
            @RequestParam("osName")
            @Parameter(description = "支持的操作系统名称", required = true)
            String osName,
            @RequestParam("osArch")
            @Parameter(description = "支持的操作系统架构", required = true)
            String osArch
    );
}