package cd.casic.ci.ticket.api;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.ci.ticket.dal.pojo.CredentialCreate;
import cd.casic.ci.ticket.dal.pojo.CredentialInfo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.*;

/**
 * @Author：mianbin
 * @Package：cd.casic.pipeline.api
 * @Project：ops-pro
 * @name：BuildCredentialResource
 * @Date：2025/03/22 15:49
 * @Filename：BuildCredentialResource
 */
@RestController
@Tag(name = "BUILD_CREDENTIAL", description = "构建-凭据资源")
@RequestMapping("/build/credentials")
public interface BuildCredentialApi {
    @Operation(summary = "构建机获取凭据")
    @GetMapping("/{credentialId}/")
    CommonResult<CredentialInfo> get(
            @Parameter(description = "项目ID", required = true)
            @RequestHeader(AUTH_HEADER_DEVOPS_PROJECT_ID)
            String projectId,
            @Parameter(description = "构建ID", required = true)
            @RequestHeader(AUTH_HEADER_DEVOPS_BUILD_ID)
            String buildId,
            @Parameter(description = "构建环境ID", required = true)
            @RequestHeader(AUTH_HEADER_DEVOPS_VM_SEQ_ID)
            String vmSeqId,
            @Parameter(description = "构建机名称", required = true)
            @RequestHeader(AUTH_HEADER_DEVOPS_VM_NAME)
            String vmName,
            @Parameter(description = "凭据ID", required = true)
            @PathVariable("credentialId")
            String credentialId,
            @Parameter(description = "Base64编码的加密公钥", required = true)
            @RequestParam("publicKey")
            String publicKey,
            @Parameter(description = "插件ID", required = true)
            @RequestHeader("X-DEVOPS-CI-TASK-ID")
            String taskId,
            @Parameter(description = "插件ID(旧版本的，为了兼容旧版本插件不用更新sdk来使用)", required = true)
            @RequestHeader("X-DEVOPS-TASK-ID")
            String oldTaskId
    );

    @Operation(summary = "构建机获取跨项目凭据")
    @GetMapping("/{credentialId}/across/")
    CommonResult<CredentialInfo> getAcrossProject(
            @Parameter(description = "项目ID", required = true)
            @RequestHeader("X-DEVOPS-PROJECT-ID")
            String projectId,
            @Parameter(description = "构建ID", required = true)
            @RequestHeader("X-DEVOPS-BUILD-ID")
            String buildId,
            @Parameter(description = "构建环境ID", required = true)
            @RequestHeader("X-DEVOPS-VM-SEQ-ID")
            String vmSeqId,
            @Parameter(description = "构建机名称", required = true)
            @RequestHeader("X-DEVOPS-VM-NAME")
            String vmName,
            @Parameter(description = "凭据ID", required = true)
            @PathVariable("credentialId")
            String credentialId,
            @Parameter(description = "项目ID", required = true)
            @RequestParam("targetProjectId")
            String targetProjectId,
            @Parameter(description = "Base64编码的加密公钥", required = true)
            @RequestParam("publicKey")
            String publicKey
    );

    @Operation(summary = "插件获取凭据")
    @GetMapping("/{credentialId}/detail")
    CommonResult<Map<String, String>> getDetail(
            @Parameter(description = "项目ID", required = true)
            @RequestHeader("X-DEVOPS-PROJECT-ID")
            String projectId,
            @Parameter(description = "构建ID", required = true)
            @RequestHeader("X-DEVOPS-BUILD-ID")
            String buildId,
            @Parameter(description = "构建环境ID", required = true)
            @RequestHeader("X-DEVOPS-VM-SEQ-ID")
            String vmSeqId,
            @Parameter(description = "构建机名称", required = true)
            @RequestHeader("X-DEVOPS-VM-NAME")
            String vmName,
            @Parameter(description = "插件ID", required = true)
            @RequestHeader("X-DEVOPS-CI-TASK-ID")
            String taskId,
            @Parameter(description = "插件ID(旧版本的，为了兼容旧版本插件不用更新sdk来使用)", required = true)
            @RequestHeader("X-DEVOPS-TASK-ID")
            String oldTaskId,
            @Parameter(description = "凭据ID", required = true)
            @PathVariable("credentialId")
            String credentialId
    );

    @Operation(summary = "新增凭据")
    @PostMapping("/{projectId}/")
    CommonResult<Boolean> create(
            @Parameter(description = "用户ID", required = true, example = "admin")
            @RequestHeader("X-DEVOPS-USER-ID")
            String userId,
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "凭据", required = true)
            CredentialCreate credential
    );

}
